﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Linq;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Clients.Hive.Views {
  [View("StateLogGanttChartList View")]
  [Content(typeof(StateLogListList), true)]
  public sealed partial class StateLogGanttChartListView : ItemView {
    public new StateLogListList Content {
      get { return (StateLogListList)base.Content; }
      set { base.Content = value; }
    }

    public StateLogGanttChartListView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      // Deregister your event handlers here
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // Register your event handlers here
    }

    #region Event Handlers (Content)
    // Put event handlers of the content here
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // Add code when content has been changed and is null
        ganttChart.Reset();
      } else {
        // Add code when content has been changed and is not null
        ganttChart.Reset();
        SetupCategories(ganttChart);

        DateTime maxValue = Content.Max(x => x.Max(y => y.DateTime));
        var upperLimit = DateTime.FromOADate(Math.Min(DateTime.Now.AddSeconds(10).ToOADate(), maxValue.AddSeconds(10).ToOADate()));

        for (int i = 0; i < Content.Count; i++) {
          for (int j = 0; j < Content[i].Count - 1; j++) {
            if(Content[i][j].State != JobState.Offline && Content[i][j].State != JobState.Finished) 
              AddData(ganttChart, i.ToString(), Content[i][j], Content[i][j+1], upperLimit);
          }
          if(Content[i].Count > 0) {
            AddData(ganttChart, i.ToString(), Content[i][Content[i].Count - 1], null, upperLimit);
          }
        }
      }
    }

    public static void SetupCategories(GanttChart ganttChart) {
      ganttChart.AddCategory(JobState.Offline.ToString(), Color.Gainsboro);
      ganttChart.AddCategory(JobState.Waiting.ToString(), Color.NavajoWhite);
      ganttChart.AddCategory(JobState.Transferring.ToString(), Color.CornflowerBlue);
      ganttChart.AddCategory(JobState.Calculating.ToString(), Color.DarkGreen);
      ganttChart.AddCategory(JobState.Finished.ToString(), Color.White);
      ganttChart.AddCategory(JobState.Aborted.ToString(), Color.Orange);
      ganttChart.AddCategory(JobState.Failed.ToString(), Color.Red);
    }

    public static void AddData(GanttChart ganttChart, string name, StateLog from, StateLog to, DateTime upperLimit) {
      DateTime until = to != null ? to.DateTime : upperLimit;
      TimeSpan duration = until - from.DateTime;
      string tooltip = string.Format("State: {0}\nDuration: {1}\n{2} - {3}", from.State, duration, from.DateTime, until);
      if (!string.IsNullOrEmpty(from.Exception))
        tooltip += "\n" + from.Exception;
      ganttChart.AddData(name, from.State.ToString(), from.DateTime, until, tooltip, false);
    }
    
    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // Enable or disable controls based on whether the content is null or the view is set readonly
    }

    #region Event Handlers (child controls)
    // Put event handlers of child controls here.
    #endregion
  }
}
